--[[Author:		Cyprias 
	License:	All Rights Reserved
	Contact:	Cyprias on Curse.com or WowAce.com

	Only Curse.com and Wowace.com have permission to host this addon.
	I have not given permission for Totem Radius to be used in any addon compilation or UI pack.]]
	
-- global lookup
local pairs_ 								= pairs
local InterfaceOptionsFrame_OpenToCategory_	= InterfaceOptionsFrame_OpenToCategory
local GetCVar_ 								= GetCVar
local select_								= select
local UnitInParty_ = UnitInParty
local UnitInRaid_ = UnitInRaid
local GetTime_ = GetTime

local GetTotemInfo_							= GetTotemInfo
local select_ 								= select
local Minimap_
local CreateFrame_ 							= CreateFrame
local unpack_ 								= unpack
local GetCVar_ 								= GetCVar
local math_deg = math.deg
local math_atan2 = math.atan2
local GetPlayerMapPosition_ = GetPlayerMapPosition


local folder, core = ...
TR = core --global table

-- Isolate the environment
local _G = getfenv(0)
setmetatable(TR, {__index = _G})
setfenv(1, TR)
-- local
title		= "Totem Radius"
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
titleFull	= title.." "..version
addonDir = "Interface\\AddOns\\TotemRadius\\"

LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0", "AceSerializer-3.0") 

commPrefix = "TR1"

Astrolabe = DongleStub("Astrolabe-0.4")

local activeTotems = {}

minimapSize = { -- size of minimap in yards.
	indoor = {
		[0] = 150,
		[1] = 120,
		[2] = 90,
		[3] = 60,
		[4] = 40,
		[5] = 25,
	},
	outdoor = {
		[0] = 233 + 1/3,
		[1] = 200,
		[2] = 166 + 2/3,
		[3] = 133 + 1/6,
		[4] = 100,
		[5] = 66 + 2/3,
	},
}

local regEvents = {
--~ 	"PLAYER_TOTEM_UPDATE",
	"MINIMAP_UPDATE_ZOOM",
	"COMBAT_LOG_EVENT_UNFILTERED",
--~ 	"UNIT_AURA",
}
db	= {}
P	= {} --db.profile

userName = ""

defaultSettings	= {
	profile = {
		totemColour= {},
	},
}

function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("TR_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	self:RegisterChatCommand("tr", "MySlashProcessorFunc")
	
	
	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, titleFull)
	
	Minimap_ = _G.Minimap
	

end

function core:OnEnable()
	P = db.profile

	for i, event in pairs_(regEvents) do 
		self:RegisterEvent(event)
	end

	self:SecureHook(Minimap_, "SetZoom", "Minimap_SetZoom")
	
	self:RegisterComm(commPrefix, "IncComm")
	
	userName = GetUnitName("player")
end

function core:OnDisable()

end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /da function brings up the UI options.	--
----------------------------------------------
	InterfaceOptionsFrame_OpenToCategory_(coreOpts)
end


--~ local AceGUI = LibStub("AceGUI-3.0")

myTotems = {}

function abc(dir)--	/run TR.abc(90)
	local cC, cZ, cX, cY = Astrolabe:GetCurrentPlayerPosition()
	
	print("a", cX, cY)
	
	local posX, posY = GetPlayerMapPosition("player");
	print("b", posX, posY)
end

local indoors = Minimap:GetZoom() == GetCVar_( "minimapInsideZoom" ) + 0;

function core:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags  = ...-- ***
	if eventType == "SPELL_SUMMON" then
		local spellID, spellName, spellSchool = select_(9,...)
		if totemInfo[spellID] then
			local tC, tZ, tX, tY
			local precise = false
			local totemSlot = totemInfo[spellID].school --We need the school to know what direction the totem spawned. (Fire's always 45deg, Earth 270deg, ect)
				
			if UnitIsUnit("player", srcName) then
				tC, tZ = Astrolabe:GetCurrentPlayerPosition()
				tX, tY = GetTotemCoords(totemHeadings[totemSlot])
				precise = true
			elseif P.trackOthersTotems == true and UnitInParty_(srcName) or UnitInRaid_(srcName) then
				tC, tZ, tX, tY = Astrolabe:GetUnitPosition(srcName)
			else
				return
			end
			
			if tC then
				AddTotem(dstGUID, spellID, srcName, tC, tZ, tX, tY, precise, totemSlot)
				if P.broadcastTotemLocation == true then
					BroadcastTotemLoc(dstGUID, spellID, tC, tZ, tX, tY)
				end
			end
		end
	elseif eventType == "UNIT_DIED" or eventType == "UNIT_DESTROYED" then
		if dstName:find("Totem") then
			RemoveTotem(dstGUID)
		end
	end
end


function core:IncComm(prefix, message, distribution, sender)
	if P.trackOthersTotems == true and sender ~= userName then
		local success, totemGUID, spellID, tC, tZ, tX, tY = core:Deserialize(message)
--~ 		Debug("IncComm",sender.." dropped a totem!")
		if success then
			AddTotem(totemGUID, spellID, sender, tC, tZ, tX, tY, true)
		end
	end
end

function BroadcastTotemLoc(totemGUID, spellID, tC, tZ, tX, tY)
	local str = core:Serialize(totemGUID, spellID, tC, tZ, tX, tY)
--~ 	Debug("BroadcastTotemLoc", str, str:len())
	core:SendCommMessage(commPrefix, str, "GUILD", nil, "ALERT")
end



function AddTotem(totemGUID, spellID, caster, tC, tZ, tX, tY, precise, totemSlot)
	if not totemInfo[spellID] then
		return
	end
	
	local radius = minimapSize[indoors and "indoor" or "outdoor"][Minimap_:GetZoom()]
	local minimapWidth = Minimap_:GetWidth()
	
	local range = totemInfo[spellID].range or 0
	if precise == false and range > spawnDistance then
		range = range - spawnDistance --We don't know the precise coords of the totem. So we remove 3 yards from the totem's range to make sure we'll be inside it. 
	end
	
	local size = (minimapWidth * range/radius)
	if not activeTotems[totemGUID] then
		local totemSlot = totemSlot or totemInfo[spellID].school --We need the school to know what direction the totem spawned. (Fire's always 45deg, Earth 270deg, ect)
		
		activeTotems[totemGUID] = {
			icon	= CreateFrame_("Button",nil,Minimap_),
			caster	= caster,
			coords	= {tC, tZ, tX, tY},
			range	= range,
			precise	= precise,--We don't know the precise coords of other people's totems.
			totemSlot = totemSlot,
			death = GetTime_() + totemInfo[spellID].duration,
		}
		activeTotems[totemGUID].icon.totem = activeTotems[totemGUID] --Add our totem table to the icon frame so it can be used in the OnUpdate script.
		activeTotems[totemGUID].icon.GUID = totemGUID
		
		SetOnUpdateScript(activeTotems[totemGUID].icon)
		
		activeTotems[totemGUID].icon:SetWidth(P.iconSize)
		activeTotems[totemGUID].icon:SetHeight(P.iconSize)
		
		activeTotems[totemGUID].icon.spellTexture = activeTotems[totemGUID].icon:CreateTexture(nil,"ARTWORK")
		activeTotems[totemGUID].icon.spellTexture:SetTexture(GetSpellTexture(totemInfo[spellID].name))
		activeTotems[totemGUID].icon.spellTexture:SetAllPoints(activeTotems[totemGUID].icon)
		
		activeTotems[totemGUID].icon.solidTexture = activeTotems[totemGUID].icon:CreateTexture(nil,"BACKGROUND")
		activeTotems[totemGUID].icon.solidTexture:SetTexture(addonDir.."texture\\Solid")
		activeTotems[totemGUID].icon.solidTexture:SetPoint("CENTER", activeTotems[totemGUID].icon);
		activeTotems[totemGUID].icon.solidTexture:SetAlpha(0.3)
		
		activeTotems[totemGUID].icon.outlineTexture = activeTotems[totemGUID].icon:CreateTexture(nil,"BACKGROUND")
		activeTotems[totemGUID].icon.outlineTexture:SetTexture(addonDir.."texture\\Outline")
		activeTotems[totemGUID].icon.outlineTexture:SetPoint("CENTER", activeTotems[totemGUID].icon);
		activeTotems[totemGUID].icon.outlineTexture:SetAlpha(1)
		
		activeTotems[totemGUID].icon.solidTexture:SetVertexColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.totemColour[totemSlot].a);
		activeTotems[totemGUID].icon.outlineTexture:SetVertexColor(0, 1, 0, 1); --first drop, colour it green.
	elseif precise == true then
		--We're already tacking the totem but another user has sent us more accurate coords.
		Debug("AddTotem", tostring(caster).." has sent us more accurate coords for "..tostring(totemInfo[spellID].name))
		activeTotems[totemGUID].coords = {tC, tZ, tX, tY}
		activeTotems[totemGUID].range = range
	end

	activeTotems[totemGUID].icon.solidTexture:SetWidth(size) -- Set these to whatever height/width is needed 
	activeTotems[totemGUID].icon.solidTexture:SetHeight(size) -- for your Texture
	
	activeTotems[totemGUID].icon.outlineTexture:SetWidth(size) -- Set these to whatever height/width is needed 
	activeTotems[totemGUID].icon.outlineTexture:SetHeight(size) -- for your Texture

	local result = Astrolabe:PlaceIconOnMinimap( activeTotems[totemGUID].icon, tC, tZ, tX, tY )
	activeTotems[totemGUID].icon:Show()
end

function RemoveTotem(totemGUID)
	if activeTotems[totemGUID] then
		activeTotems[totemGUID].icon:Hide()
		activeTotems[totemGUID].icon = nil
		activeTotems[totemGUID] = nil
	end
end

function SetOnUpdateScript(frame)
	frame.lastUpdate = 0
--~ 	frame.lastDist = 0
	frame:SetScript("OnUpdate", function(self, elapsed)
		self.lastUpdate = self.lastUpdate + elapsed
		if self.lastUpdate > 0.1 then
			self.lastUpdate = 0
			
--~ 			Debug("OnUpdate", GetTime_(), self.totem.death, GetTime_() >= self.totem.death)
			if GetTime_() >= self.totem.death then
				RemoveTotem(self.GUID)
--~ 				echo("Removing totem due to age")
				return
			end
			
			local cC, cZ, cX, cY = Astrolabe:GetCurrentPlayerPosition()
			local pC, pZ, pX, pY = unpack_(self.totem.coords)
			local dist = Astrolabe:ComputeDistance(  pC, pZ, pX, pY, cC, cZ, cX, cY )
--~ 					local hasBuff = UnitBuff("player", self.totem.buff)
			if dist < self.totem.range then --hasBuff and 
				self.outlineTexture:SetVertexColor(0, 1, 0);
			else
				if dist > self.totem.range then
					self.outlineTexture:SetVertexColor(1, 0, 0);
				else
					self.outlineTexture:SetVertexColor(0, 1, 0);
				end
				
			end

			if Astrolabe:IsIconOnEdge(self) then
				if Astrolabe:GetDistanceToIcon( self ) < P.minimapIconRange then
					self:SetAlpha(1)
				else
					self:SetAlpha(0)
				end
			else
				self:SetAlpha(1)
			end
		end
	end)
end

local lastZoom = 0
local ignoreZoom = false
function core:MINIMAP_UPDATE_ZOOM(event, ...)
	if ignoreZoom == true then --Keeps us from entering a endless loop due to SetZoom(zoom>0...) later on.
		return
	end
	local zoom = Minimap:GetZoom();
	if zoom ~= lastZoom then --Keeps us from processing the same zoom info repeatedly due to SetZome(zoom) later on. 
		lastZoom = zoom
		if ( GetCVar_( "minimapZoom" ) == GetCVar_( "minimapInsideZoom" ) ) then -- Indeterminate case
			ignoreZoom = true
 			Minimap:SetZoom( zoom > 0 and zoom - 1 or zoom + 1 ); -- Any change to make the cvars unequal
			ignoreZoom = false
		end
		indoors = Minimap:GetZoom() == GetCVar_( "minimapInsideZoom" ) + 0;
		Minimap:SetZoom(zoom);

		local range, size
		local radius = minimapSize[indoors and "indoor" or "outdoor"][Minimap_:GetZoom()]
		local minimapWidth = Minimap_:GetWidth()
--~ 		for name, data in pairs_(myTotems) do 
		for GUID, data in pairs_(activeTotems) do 
			size = (minimapWidth * data.range/radius)
			
--~ 			Debug(event, indoors, radius, minimapWidth, size)
			
			data.icon.solidTexture:SetWidth(size) -- Set these to whatever height/width is needed 
			data.icon.solidTexture:SetHeight(size) -- for your Texture
			
			data.icon.outlineTexture:SetWidth(size) -- Set these to whatever height/width is needed 
			data.icon.outlineTexture:SetHeight(size) -- for your Texture
		end
	end
end

function core:Minimap_SetZoom(...)
	core:MINIMAP_UPDATE_ZOOM("MINIMAP_UPDATE_ZOOM")
end

function OnEdgeChanged(...)
--~ 	for name, data in pairs_(myTotems) do 
	for GUID, data in pairs_(activeTotems) do 
		if(Astrolabe:IsIconOnEdge(data.icon)) then
			data.icon.solidTexture:SetAlpha(0)
			data.icon.outlineTexture:SetAlpha(0)
		else
			data.icon.solidTexture:SetAlpha(P.totemColour[data.totemSlot].a)
			data.icon.outlineTexture:SetAlpha(1)
			data.icon:SetAlpha(1)
--~ 			data.icon:Show()
		end
	end
end
Astrolabe:Register_OnEdgeChanged_Callback( OnEdgeChanged, "xyz" )
